#prepare for cross validation

get_cross_validated_penalty_parameters <- function(predictor,
                                                   predicted,
                                                   penalization,
                                                   ridge_penalty,
                                                   nonzero,
                                                   nr_subsets,
                                                   max_iterations,
                                                   tolerance,
                                                   parallel_CV = parallel_CV
                                                   ){
  
  shuffled <-  get_split_sets(X = predictor, Y = predicted, nr_subsets = nr_subsets)
  
  X.sampled     <-   shuffled$X.sampled
  Y.sampled     <-   shuffled$Y.sampled
  label         <-   shuffled$labels
  
  if (parallel_CV){
    cv_results <- get_parallel_cv(X = X.sampled,
                                           Y = Y.sampled, 
                                           lambdas = ridge_penalty, 
                                           non_zeros = nonzero, 
                                           label = label,
                                           penalization = penalization,
                                           max_iterations = max_iterations,
                                           tolerance = tolerance)
  
  } else {
    cv_results <- get_non_parallel_cv(X = X.sampled,
                        Y = Y.sampled,
                        lambdas = ridge_penalty,
                        non_zeros = nonzero,
                        label = label,
                        penalization = penalization,
                        max_iterations = max_iterations,
                        tolerance = tolerance)
    
  }

  # cv_results <- switch(parallel_CV,
  #                      TRUE = {
  #                        get_parallel_cv(X = X.sampled,
  #                                        Y = Y.sampled, 
  #                                        lambdas = ridge_penalty, 
  #                                        non_zeros = nonzero, 
  #                                        label = label,
  #                                        penalization = penalization,
  #                                        tolerance = tolerance)
  #                      },
  #                      FALSE = {
  #                        get_non_parallel_cv(X = X.sampled,
  #                                            Y = Y.sampled,
  #                                            lambdas = ridge_penalty,
  #                                            non_zeros = nonzero,
  #                                            label = label,
  #                                            penalization = penalization,
  #                                            tolerance = tolerance)
  #                      }
  #                     
  #                      
  #                      
  # )

  a = cv_results$mean_abs_cors[,3]
  
  best_values     <- cv_results$mean_abs_cors[which.max(a),]
  
  best_ridge   <- best_values[1]
  best_nonzero   <- best_values[2]

  
  #**********************
  result <- list(cv_results = cv_results,
                 best_ridge = best_ridge,
                 best_nonzero = best_nonzero
  )
  
  
  result
  
}